@@ -12,13 +12,16 @@
 // See the License for the specific language governing permissions and
 // limitations under the License.!
 
-#include "builder.h"
 #include <algorithm>
 #include <functional>
 #include <utility>
-#include "filesystem.h"
 
-#include "config.h"
+#include "builder.h"
+#include "filesystem.h"
+#include "third_party/absl/strings/str_join.h"
+#include "third_party/absl/strings/str_replace.h"
+#include "third_party/absl/strings/str_split.h"
+#include "third_party/absl/strings/strip.h"
 
 #ifdef ENABLE_NFKC_COMPILE
 #include <unicode/errorcode.h>
@@ -27,7 +30,7 @@
 #include <unicode/numfmt.h>
 #include <unicode/rbnf.h>
 #include <unicode/utypes.h>
-#endif
+#endif  // ENABLE_NFKC_COMPILE
 
 #include <set>
 
@@ -51,14 +54,7 @@ Builder::Chars UnicodeNormalize(UNormalizationMode mode,
   const std::string utf8 = string_util::UnicodeTextToUTF8(input);
   CHECK(!utf8.empty());
 
-  icu::UnicodeString ustr;
-  const size_t utf8_length = utf8.size();
-  UChar *utf16 = ustr.getBuffer(utf8.size() + 1);
-  int32 utf16_length = 0;
-  icu::ErrorCode icuerrorcode;
-  u_strFromUTF8Lenient(utf16, ustr.getCapacity(), &utf16_length, utf8.data(),
-                       utf8_length, icuerrorcode);
-  ustr.releaseBuffer(utf16_length);
+  icu::UnicodeString ustr = icu::UnicodeString::fromUTF8(utf8.c_str());
 
   UErrorCode status = U_ZERO_ERROR;
   icu::UnicodeString dst;
@@ -285,7 +281,7 @@ util::Status Builder::GetPrecompiledCharsMap(const std::string &name,
       return util::OkStatus();
     }
   }
-  return util::StatusBuilder(util::error::NOT_FOUND)
+  return util::StatusBuilder(util::StatusCode::kNotFound, GTL_LOC)
          << "No precompiled charsmap is found: " << name;
 }
 
@@ -490,18 +486,19 @@ util::Status Builder::LoadCharsMap(absl::string_view filename,
   std::string line;
   chars_map->clear();
   while (input->ReadLine(&line)) {
-    auto fields = string_util::SplitPiece(line, "\t", true /* allow empty*/);
+    std::vector<std::string> fields =
+        absl::StrSplit(line, "\t", absl::AllowEmpty());
     CHECK_GE(fields.size(), 1);
     if (fields.size() == 1) fields.push_back("");  // Deletion rule.
     std::vector<char32> src, trg;
-    for (auto &s : string_util::SplitPiece(fields[0], " ")) {
+    for (auto s : absl::StrSplit(fields[0], " ")) {
       if (s.empty()) continue;
-      string_util::ConsumePrefix(&s, "U+");
+      absl::ConsumePrefix(&s, "U+");
       src.push_back(string_util::HexToInt<char32>(s));
     }
-    for (auto &s : string_util::SplitPiece(fields[1], " ")) {
+    for (auto s : absl::StrSplit(fields[1], " ")) {
       if (s.empty()) continue;
-      string_util::ConsumePrefix(&s, "U+");
+      absl::ConsumePrefix(&s, "U+");
       trg.push_back(string_util::HexToInt<char32>(s));
     }
     CHECK_OR_RETURN(!src.empty());
@@ -528,15 +525,13 @@ util::Status Builder::SaveCharsMap(absl::string_view filename,
       trg.push_back(string_util::IntToHex(v));
       trgu.push_back(v);
     }
-    std::string line = string_util::Join(src, " ") + "\t" +
-                       string_util::Join(trg, " ") + "\t# " +
+    std::string line = absl::StrJoin(src, " ") + "\t" +
+                       absl::StrJoin(trg, " ") + "\t# " +
                        string_util::UnicodeTextToUTF8(c.first) + " => " +
                        string_util::UnicodeTextToUTF8(c.second);
-    line = string_util::StringReplace(line, "\b", " ", true);
-    line = string_util::StringReplace(line, "\v", " ", true);
-    line = string_util::StringReplace(line, "\f", " ", true);
-    line = string_util::StringReplace(line, "\n", " ", true);
-    line = string_util::StringReplace(line, "\r", " ", true);
+    line = absl::StrReplaceAll(
+        line,
+        {{"\b", " "}, {"\v", " "}, {"\f", " "}, {"\n", " "}, {"\r", " "}});
     output->WriteLine(line);
   }
 
